% ML_ARFIMAmd - Minimum Distance Estimation of ARFIMA model as in Mayoral (2007, EctJ)
%
% [d phi psi]=mdar1me(series,p,q,d0);
%   d   - Fractionally integrated parameter
%   phi - autoregressive polynomial
%   psi - moving average polynomial              (currently unavailable) 
%   y   - variable of interest
%   p   - number of lags for phi
%   q   - number of lags for psi                 (currently unavailable) 
%   d0  - starting value for d (your best guess)
%
% REFERENCE: Laura Mayoral "Minimum distance estimation of stationary and non-stationary ARFIMA processes"
%            Econometrics Journal (2007), volume 10, pp. 124148.

% This file is a generalization of mdarime and varime written by Laura
% Mayoral

function [d phi]=ML_ARFIMAmd(y,p,q,d0)

q=0; % The MA estimation is not programmed yet 
if p==0; x0=gph(y); else x0=[gph(y) PartialCorr(y,p)']; end
% x0=[d0 zeros(1,p)];
options=optimset('Display','off','TolFun',1.0e-12,'TolX',1.0e-12);
v=fminsearch(@(theta) var1me(theta,p,q,y),x0,options); % minimization

% A=[-1; 1];  A=[A zeros(2,p)]; B=[-.2; .49];                                    % constraint for d
% options=optimset('Display','off','TolFun',1.0e-12,'TolX',1.0e-12);          % options for minimization
% v=fmincon(@(theta) var1me(theta,p,q,y),x0,A,B,[],[],[],[],[],options);      % minimization


d=v(1); 
phi=v(2:p+1);
% psi=theta(p+3:end);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% This is the criterion function of the minimum distance estimation method for the ARFIMA(1,d,0) process with unknown mean;
% arguments: p: parameters, (d and ar1); series: data

function[v]=var1me(theta,p,q,series)

d=theta(1); 
phi=theta(2:p+1);
% psi=theta(p+3:end);
n=length(series);
m=fix(d);
pq=max(p,q);
M=max(round(n^(1/4)),(p+q+1));
% disp([M m theta])
% M=p+q+1;
if m>=0
    yd=ML_DiffFrac(series,m);  
    yd=yd-mean(yd);    
    delta=d-m;
    yd=ML_DiffFrac(yd,delta); 
    et=ML_lag(yd,p,0)*[1 -phi]';
    a=AutoCorr(et,M);                                                       % criterion function    
    v=(a'*a);
else
    series=series-mean(series);
    yd=ML_DiffFrac(series,d) ;
    et=ML_lag(yd,p,0)*[1 -phi]';
    a=AutoCorr(et,M);                                                       % criterion function
    v=(a'*a);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% AC=AutoCorr(Y,g) - Compute autocorrelation of Y up to order g

function AC=AutoCorr(Y,g)
AC=zeros(g,1);
t = size(Y,1);    
y = ML_center(Y);
s2 = y'*y;
for j=1:g;
    x1=y(j+1:t);    
    x2=y(1:t-j);    
    AC(j,1)=(x1'*x2)/s2;    
    clear x1 x2    
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% PC=PartialCorr(Y,g) - Compute Partial Auto Vorrelation of Y up to order g

function PC=PartialCorr(Y,g)

for ii=1:g;    
    b = ML_ols(Y(ii+1:end,:),ML_lag(Y,ii),1);          
    PC(ii,1)=b(ii+1,1);    
end

